/*

./test_srv -- -ORBInitRef NameService=corbaloc:iiop:127.0.0.1:2809/NameService

 */

#include <rtt/os/main.h>

#include <rtt/TaskContext.hpp>
#include <rtt/OperationCaller.hpp>

#include <rtt/transports/corba/TaskContextServer.hpp>
#include <rtt/transports/corba/TaskContextProxy.hpp>

using namespace RTT::corba;
using namespace RTT;

int ORO_main(int argc, char** argv)
{

  // Setup Corba:
  TaskContextProxy::InitOrb(argc, argv);
  
  // Get a pointer to the component above
  TaskContext* component = TaskContextProxy::Create( "Foo" );

  // Interface it:
  RTT::OperationCaller<double(void)>      getDiskUsage;
  RTT::SendHandle     <double(void)>      getDiskUsageHandle;

  getDiskUsage = component->getOperation("getDiskUsage");
  if(!getDiskUsage.ready())
    {
      fprintf(stderr, "no \"getDiskUsage\" operation\n");
    }

  while(1)
    {
#define OPT 2

#if (OPT == 1)
      fprintf(stderr, "val = %f\n", getDiskUsage());
#elif (OPT == 2)
      getDiskUsageHandle = getDiskUsage.send();
      double ret;
      SendStatus ss = getDiskUsageHandle.collect(ret);
      if (ss != SendSuccess) 
        {
          fprintf(stderr, "Execution of getDiskUsageHandle failed.");
        }
      fprintf(stderr, "val = %f\n", ret);
#elif (OPT == 3)
      getDiskUsageHandle = getDiskUsage.send();  
      double ret;
      while (getDiskUsageHandle.collectIfDone(ret) == SendNotReady )
        {
          sleep(1);
        }
      fprintf(stderr, "val = %f\n", getDiskUsage());
#elif (OPT == 4)
      if(!getDiskUsageHandle.ready())
        {
          fprintf(stderr, "%d\n", __LINE__);
          getDiskUsageHandle = getDiskUsage.send();
        }
      else
        {
          fprintf(stderr, "%d\n", __LINE__);
          double ret;
          switch(getDiskUsageHandle.collectIfDone(ret))
            {
            case RTT::SendFailure:
              getDiskUsageHandle = getDiskUsage.send();
              break;
            case RTT::SendSuccess:
              fprintf(stderr, "val = %f\n", ret);
              break;
            default:
              break;
            }
        }
      fprintf(stderr, "%d\n", __LINE__);
#endif

      sleep(1);
    }

  // Cleanup Corba:
  TaskContextProxy::DestroyOrb();

  return 0;
} 
